#cython: language_level=3

from .capi.obiview      cimport obi_column_get_obifloat_with_elt_name_in_view, \
                                obi_column_get_obifloat_with_elt_idx_in_view, \
                                obi_column_set_obifloat_with_elt_name_in_view, \
                                obi_column_set_obifloat_with_elt_idx_in_view
from .capi.obierrno     cimport obi_errno
from .capi.obitypes     cimport OBIFloat_NA, obifloat_t, byte_t
from .capi.obidmscolumn cimport obi_float_column_greater_than   # TODO need to be in view...


from obitools3.utils cimport str2bytes


cdef class OBIDMS_column_float(OBIDMS_column):

    cpdef object get_line(self, index_t line_nb):
        cdef obifloat_t value
        cdef object result
        value = obi_column_get_obifloat_with_elt_idx_in_view(self.view, (self.pointer)[0], line_nb, 0)
        if obi_errno > 0 :
            raise IndexError(line_nb)
        if value == OBIFloat_NA :
            result = None
        else :
            result = <double> value
        return result

    cpdef set_line(self, index_t line_nb, object value):
        if value is None :
            value = OBIFloat_NA
        if obi_column_set_obifloat_with_elt_idx_in_view(self.view, (self.pointer)[0], line_nb, 0, <obifloat_t> value) < 0:
            raise Exception("Problem setting a value in a column")

    cpdef list bool_list_from_compare(self, obifloat_t value, int comp):
        cdef list    result_p
        cdef byte_t* result_c
        cdef int     i
        result_p = []
        if comp == 4 :
            result_c = obi_float_get_bool_list_from_compare_in_view(self.view, (self.pointer)[0], value, GREATER_THAN)
        else :
            raise Exception("Comparison not handled")
        for i in range(self.get_nb_lines_used()) :
            result_p.append(<int> result_c[i])
            print(result_p[i])
        free(result_c)
        return result_p



cdef class OBIDMS_column_multi_elts_float(OBIDMS_column_multi_elts):

    cpdef object get_item(self, index_t line_nb, str element_name):
        cdef obifloat_t value
        cdef object result
        value = obi_column_get_obifloat_with_elt_name_in_view(self.view, (self.pointer)[0], line_nb, str2bytes(element_name))
        if obi_errno > 0 :
            raise IndexError(line_nb, element_name)
        if value == OBIFloat_NA :
            result = None
        else :
            result = <double> value
        return result

    cpdef object get_line(self, index_t line_nb) :
        cdef obifloat_t value
        cdef object value_in_result
        cdef dict result
        cdef index_t i
        cdef bint all_NA
        result = {}
        all_NA = True
        for i in range(self.nb_elements_per_line) :
            value = obi_column_get_obifloat_with_elt_idx_in_view(self.view, (self.pointer)[0], line_nb, i)
            if obi_errno > 0 :
                raise IndexError(line_nb)
            if value == OBIFloat_NA :
                value_in_result = None
            else :
                value_in_result = <double> value
            result[self.elements_names[i]] = value_in_result
            if all_NA and (value_in_result is not None) :
                all_NA = False
        if all_NA :
            result = None
        return result
   
    cpdef set_item(self, index_t line_nb, str element_name, object value):
        if value is None :
            value = OBIFloat_NA
        if obi_column_set_obifloat_with_elt_name_in_view(self.view, (self.pointer)[0], line_nb, str2bytes(element_name), <obifloat_t> value) < 0:
            raise Exception("Problem setting a value in a column")
 