#cython: language_level=3

from obitools3.utils   cimport bytes2str, str2bytes

from .capi.obitaxonomy cimport obi_read_taxonomy, \
                               obi_close_taxonomy, \
                               obi_taxo_get_taxon_with_taxid

from ._obidms cimport OBIDMS

from cpython.pycapsule cimport PyCapsule_New, PyCapsule_GetPointer


cdef class OBI_Taxonomy :

    def __init__(self, OBIDMS dms, str name) :
        
        self.name = name
        self.pointer = obi_read_taxonomy(dms.pointer, str2bytes(name), True)    # TODO discuss
    
    
    def __getitem__(self, object ref):
        
        cdef ecotx_t* taxon_p
        cdef object taxon_capsule
        
        if type(ref) == int :
            taxon_p = obi_taxo_get_taxon_with_taxid(self.pointer, ref)
            taxon_capsule = PyCapsule_New(taxon_p, NULL, NULL)
            return OBI_Taxon(taxon_capsule)
    
         
    cpdef close(self) :
        if (obi_close_taxonomy(self.pointer) < 0) :
            raise Exception("Error closing the taxonomy")
        

cdef class OBI_Taxon :    # dict subclass?

    def __init__(self, object taxon_capsule) :
        
        cdef ecotx_t* taxon
        
        taxon = <ecotx_t*> PyCapsule_GetPointer(taxon_capsule, NULL)   
        self.pointer = taxon
        self.taxid = taxon.taxid
        self.rank = taxon.rank
        self.farest = taxon.farest
        self.parent = taxon.parent
        self.name = bytes2str(taxon.name)
        
    cpdef int32_t taxid(self):
        return self.taxid
    
    cpdef int32_t rank(self):
        return self.rank
    
    cpdef int32_t farest(self):
        return self.farest
    
    cpdef OBI_Taxon parent(self):
        cdef object parent_capsule
        parent_capsule = PyCapsule_New(self.parent, NULL, NULL)
        return OBI_Taxon(parent_capsule)
    
